@@ -1,5 +1,5 @@
 /* GMM (C) 2018 Stephane Charette <stephanecharette@gmail.com>
- * $Id: TabCanvas.cpp 2687 2018-12-06 21:15:40Z stephane $
+ * $Id: TabCanvas.cpp 2688 2018-12-07 21:20:05Z stephane $
  */
 
 #include "GMM.hpp"
@@ -15,12 +15,14 @@ TabCanvas::TabCanvas(const std::string & name) :
 	opc_register_pkg(cfg().get_str(prefix + "_opc_pkg_register") + "="),
 	tab_is_active(false),
 	export_image(cfg().get_bool(prefix + "_export_image")),
+	time_since_gmm_started(std::time(nullptr)),
 	time_since_kiln_changed(std::time(nullptr)),
 	time_since_run_changed(std::time(nullptr)),
 	time_since_pkg_changed(std::time(nullptr)),
 	time_since_last_board(std::time(nullptr)),
 	time_since_last_board_warning(0),
 	number_of_boards_when_warning_was_last_issued(0),
+	warning_message_is_already_on_screen(false),
 	timer_frequency_in_milliseconds(cfg().get_int(prefix + "_chart_redraw_ms")),
 	comm_handle(INVALID_HANDLE_VALUE),
 	current_kiln(0),
@@ -249,6 +251,9 @@ void TabCanvas::periodic_update(void)
 
 		std::string tooltip;
 
+		tooltip = "GMM started " + Lox::approximateTime(time_since_gmm_started);
+		percentages[number_of_buckets - 1].setTooltip(tooltip);
+
 		tooltip = "the kiln was last updated " + Lox::approximateTime(time_since_kiln_changed);
 		kiln_num_label	.setTooltip(tooltip);
 		kiln_num_editor	.setTooltip(tooltip);
@@ -266,30 +271,47 @@ void TabCanvas::periodic_update(void)
 		run_number_of_boards_editor.setTooltip(tooltip);
 	}
 
-	if (cfg().get_bool("warn_if_package_contains_too_many_boards"))
+	if (warning_message_is_already_on_screen == false)
 	{
-		const size_t number_of_boards_in_package = pkg_doubles.size();
-		if (number_of_boards_in_package > cfg().get_int("warn_board_threshold"))
+		if (cfg().get_bool("warn_if_package_contains_too_many_boards"))
 		{
-			if (number_of_boards_in_package > number_of_boards_when_warning_was_last_issued)
+			const size_t number_of_boards_in_package = pkg_doubles.size();
+			if (number_of_boards_in_package > cfg().get_int("warn_board_threshold"))
 			{
-				if (std::time(nullptr) - time_since_last_board_warning > cfg().get_int("warn_seconds_between_warnings"))
+				if (number_of_boards_in_package > number_of_boards_when_warning_was_last_issued)
 				{
-					const std::string name = cfg().get_str(prefix + "_name");
-					const std::string text =
-						"The number of boards for " + name + " is unusually high.\r\n"
-						"\r\n"
-						"- Run: #"		+ run_editor.getText().toStdString()			+ "\r\n"
-						"- Pkg: #"		+ pkg_editor.getText().toStdString()			+ "\r\n"
-						"- Boards: "	+ std::to_string(number_of_boards_in_package)	+ "\r\n"
-						"\r\n"
-						"Please verify that the GMM OPC Console is still running and communication with RSLinx has not been lost.";
+					if (std::time(nullptr) - time_since_last_board_warning > cfg().get_int("warn_seconds_between_warnings"))
+					{
+						const Time now = Time::getCurrentTime();
+						const std::string name = cfg().get_str(prefix + "_name");
+						const std::string text =
+							"The number of boards for " + name + " is unusually high.\r\n"
+							"\r\n"
+							"- Run: #"		+ run_editor.getText().toStdString()			+ "\r\n"
+							"- Pkg: #"		+ pkg_editor.getText().toStdString()			+ "\r\n"
+							"- Boards: "	+ std::to_string(number_of_boards_in_package)	+ "\r\n"
+							"- Date: "		+ now.formatted("%Y-%b-%d").toStdString()		+ "\r\n"
+							"- Time: "		+ now.formatted("%H:%M:%S").toStdString()		+ "\r\n"
+							"\r\n"
+							"Please verify that the GMM OPC Console is still running and communication with RSLinx has not been lost.";
 
-					LOG_MSG("prefix=" << prefix << ": " << text);
+						LOG_MSG("prefix=" << prefix << ": " << text);
 
-					AlertWindow::showMessageBox(AlertWindow::AlertIconType::WarningIcon, "GMM - " + name, text);
-					time_since_last_board_warning = std::time(nullptr);
-					number_of_boards_when_warning_was_last_issued = number_of_boards_in_package;
+						// Create the message window asynchronously so the main GUI thread continues to run.
+						// This will ensure we update the fields as needed, and continue to calculate the
+						// average, standard deviation, etc.  The message boxes are modal so the user will be
+						// forced to dismiss them before changing tabs.
+
+						warning_message_is_already_on_screen = true;
+						number_of_boards_when_warning_was_last_issued = number_of_boards_in_package;
+
+						AlertWindow::showMessageBoxAsync(AlertWindow::AlertIconType::WarningIcon, "GMM - " + name, text, "", nullptr,
+							ModalCallbackFunction::create([&](int rc)
+							{
+								warning_message_is_already_on_screen = false;
+								time_since_last_board_warning = std::time(nullptr);
+							}) );
+					}
 				}
 			}
 		}
